/*
 * При обновлении не перезаписываются файлы, рядом с которыми лежит файл
 * с таким же именем плюс суффиксом '.orig'.
 * Также не перезаписываются классы в /WEB-INF/classes/, которые не находятся в поддиректории ru/bitel/.
 * /WEB-INF/classes/ru/bitel/ перезаписывается всегда.
 */
import java.net.*;
import java.nio.file.*;
import java.util.zip.*;
import java.util.jar.*;

import ru.bitel.common.io.*;

import org.apache.commons.net.ftp.*;
import org.apache.commons.io.*;

System.out.println( "Starting update MyBGBilling" );

File deploymentsDir = new File( (String) System.getProperty( "jboss.server.config.dir", "notFoundNotExists" ) );

if( !deploymentsDir.exists() ) {

	deploymentsDir = new File( "../standalone/deployments" );

	if( !deploymentsDir.exists() ) {
	
		deploymentsDir = new File( "standalone/deployments" );
	}
} else {

	deploymentsDir = new File( deploymentsDir.getParentFile(), "deployments" );
}

File configurationDir = new File( deploymentsDir.getParentFile(), "configuration" );
File binDir = new File( deploymentsDir.getParentFile().getParentFile(), "bin" );
File warDir = new File( deploymentsDir, "MyBGBilling.war" );

String version = "7.2";

try {
    version = new Manifest( new FileInputStream( new File( warDir, "WEB-INF/classes/META-INF/MANIFEST.MF" ) ) )
    	.getMainAttributes()
    	.getValue( "Specification-Version" );
} catch(Exception ex) {
	ex.printStackTrace();                     
}

URI uri = URI.create( "ftp://ftp.bgbilling.ru/pub/bgbilling/${version}/" );

println ("Downloading MyBGBilling.war from " + uri);

File distribution = File.createTempFile( "mybgbilling-distr", ".zip" );
distribution.deleteOnExit();

FTPClient ftpClient = new FTPClient();
ftpClient.with {
    connect( uri.getHost() );
    enterLocalPassiveMode();
    login( "anonymous", "" );
    setFileType( FTP.BINARY_FILE_TYPE );
    changeWorkingDirectory( uri.getPath() );
    
    distribution.withOutputStream { ostream -> retrieveFile "MyBGBilling.war", ostream }
    
    disconnect();
}

if(distribution.length()>10000000) {                                                                                                                             
	println "Download OK"                                                                                                                                            
} else {                                                                                                                                                         
	println "Download FAILED";                                                                                                                                       
	return;                                                                                                                                                          
} 

/*
URL url = new URL('https://bgbilling.ru/download/${version}/MyBGBilling.war');

println ("Downloading MyBGBilling.war from " + url);

File distribution = File.createTempFile( "mybgbilling-distr", ".zip" );
url.withInputStream{ i -> distribution.withOutputStream{ it << i }};
distribution.deleteOnExit();

println "Download OK"
*/

// check
ZipFile zipfile = null;
try {
    zipfile = new ZipFile(distribution);
} catch (IOException e) {
	println "Check ZIP file FAILED";
    return;
} finally {
    try {
        if (zipfile != null) {
            zipfile.close();
            zipfile = null;
        }
    } catch (IOException e) {
    }
}


ZipInputStream zis = new ZipInputStream( new FileInputStream( distribution ) );

// check
for( ZipEntry ze; (ze = zis.getNextEntry()) != null; ) {
	
}

zis.close();


println "Check ZIP OK"


File classesDir = new File( warDir, "WEB-INF/classes/ru/bitel" );
classesDir.deleteDir();
classesDir.mkdirs();


zis = new ZipInputStream( new FileInputStream( distribution ) );

for( ZipEntry ze; (ze = zis.getNextEntry()) != null; ) {

	String name = ze.getName();
	
	if( ze.isDirectory() ) {
		new File( warDir, name ).mkdirs();
		
		continue;
	}
	
	if( name.startsWith("WEB-INF/classes/ru/bitel") ) {
		File file = new File( warDir, name );
		file.getParentFile().mkdirs();
		FileOutputStream os = new FileOutputStream( file );
		IOUtils.transfer( zis, os, 8192 );
		os.close();
		
		continue;
	}
	
	File file = File.createTempFile( "zip-entry", ".tmp" );
	FileOutputStream os = new FileOutputStream( file );
	IOUtils.transfer( zis, os, 8192 );
	os.close();
	file.deleteOnExit();
	
	File oldFile;
	
	if( name.startsWith( "WEB-INF/mybgbilling-" ) && name.endsWith( ".groovy" ) ) {
		continue;
	}
	
	if( name.startsWith( "WEB-INF/defaults" ) && name.endsWith( "mybgbilling-menu.groovy" ) ) {
	
		oldFile = new File( warDir, "WEB-INF/mybgbilling-menu.groovy.orig" );
		if( oldFile.exists() ) {
			if( !FileUtils.contentEquals( oldFile, file ) ) {
				println "Replace $oldFile"
				
				IOUtils.transfer( new FileInputStream( file ), new FileOutputStream( oldFile ), 8192 );
			}
		} else {
			oldFile = new File( warDir, "WEB-INF/mybgbilling-menu.groovy" );
			File oldBakFile = new File( warDir, "WEB-INF/mybgbilling-menu.groovy.0bak" );
			if( !oldBakFile.exists() ) {
				IOUtils.transfer( new FileInputStream( oldFile ), new FileOutputStream( oldBakFile ), 8192 );
			}
			
			if( !FileUtils.contentEquals( oldFile, file ) ) {
				println "Replace $oldFile"
				
				IOUtils.transfer( new FileInputStream( file ), new FileOutputStream( oldFile ), 8192 );
			}
		}
	}
	

	oldFile = new File( warDir, name + ".orig" );
	if( oldFile.exists() ) {
		if( !FileUtils.contentEquals( oldFile, file ) ) {
			println "Replace $oldFile"
			
			IOUtils.transfer( new FileInputStream( file ), new FileOutputStream( oldFile ), 8192 );
		}
	} else {
	
		if( name.endsWith( ".orig" ) ) { 
			continue; 
		}
	
		oldFile = new File( warDir, name );
		if( oldFile.exists() ) {
			if( !FileUtils.contentEquals( oldFile, file ) ) {
				println "Replace $oldFile"
				
				IOUtils.transfer( new FileInputStream( file ), new FileOutputStream( oldFile ), 8192 );
			} else {
				
				continue;
			}
		} else {
			println "Add $oldFile"
			
			file.getParentFile().mkdirs();
			IOUtils.transfer( new FileInputStream( file ), new FileOutputStream( oldFile ), 8192 );
		}
		
		if( name.startsWith( "WEB-INF/script/files" ) && name.endsWith( ".sh" ) && !name.endsWith( "setenv.sh" ) ) {
			
			def newPath = Paths.get( binDir.getPath(), oldFile.getName() );
			println "Replace $newPath"
			Files.copy( Paths.get( oldFile.getPath() ), newPath, java.nio.file.StandardCopyOption.REPLACE_EXISTING );
			
		} else if( name.startsWith( "WEB-INF/script/files" ) && name.endsWith( ".conf.groovy" ) ) {
			
			def newPath = Paths.get( configurationDir.getPath(), oldFile.getName() );
			println "Replace $newPath"
			Files.copy( Paths.get( oldFile.getPath() ), newPath, java.nio.file.StandardCopyOption.REPLACE_EXISTING );
		}
	}
}